package org.netkernel.client.http.endpoint;

import org.netkernel.client.http.representation.HttpClientConfig;
import org.netkernel.client.http.representation.IHttpClientConfig;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;

public class HttpConfigTransreptor extends StandardTransreptorImpl
{	
	public HttpConfigTransreptor()
	{	this.declareThreadSafe();
		this.declareToRepresentation(IHttpClientConfig.class);
	}

	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	
		boolean followRedirects=true;
		long acceptableContentLength=-1;
		int connectionsPerHost=4;
		int totalConnections=10;
		int stateExpirationTime=600;
		int retryAttempts=3;
		int connectTimeout=2000;
		int timeout=5000;
		String proxyHost=System.getProperty("http.proxyHost");
		int proxyPort=80;
		String sproxyPort=System.getProperty("http.proxyPort");
		if(sproxyPort!=null)
		{	proxyPort=Integer.parseInt(sproxyPort);
		}
		IHDSNode cfg=aContext.sourcePrimary(IHDSNode.class);
		cfg=cfg.getFirstNode("/config");
		if(cfg.getFirstNode("followRedirects")!=null)
		{	String temp=cfg.getFirstValue("followRedirects").toString();
			followRedirects=temp.startsWith("t") || temp.equals("1");
		}
		if(cfg.getFirstNode("retryAttempts")!=null)
		{	String temp=cfg.getFirstValue("retryAttempts").toString();
			retryAttempts=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("maxConnectionsPerHost")!=null)
		{	String temp=cfg.getFirstValue("maxConnectionsPerHost").toString() ;
			connectionsPerHost=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("maxTotalConnections")!=null)
		{	String temp=cfg.getFirstValue("maxTotalConnections").toString();
			totalConnections=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("maxAcceptableContentLength")!=null)
		{	String temp=cfg.getFirstValue("maxAcceptableContentLength").toString();
			acceptableContentLength=Long.parseLong(temp);
		}
		if(cfg.getFirstNode("stateExpirationTime")!=null)
		{	String temp=cfg.getFirstValue("stateExpirationTime").toString();
			stateExpirationTime=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("connectTimeout")!=null)
		{	String temp=cfg.getFirstValue("connectTimeout").toString();
			connectTimeout=Integer.parseInt(temp);
		}
		if(cfg.getFirstNode("timeout")!=null)
		{	String temp=cfg.getFirstValue("timeout").toString();
			timeout=Integer.parseInt(temp);
		}
		if(cfg.getFirstValue("proxyHost")!=null)
		{	proxyHost=cfg.getFirstValue("proxyHost").toString();
		}
		if(cfg.getFirstValue("proxyPort")!=null)
		{	String temp=cfg.getFirstValue("proxyPort").toString();
			proxyPort=Integer.parseInt(temp);
		}
		
		HttpClientConfig ca=new HttpClientConfig(
			followRedirects,
			acceptableContentLength,
			connectionsPerHost,
			totalConnections,
			stateExpirationTime,
			retryAttempts,
			connectTimeout,
			timeout,
			proxyHost, 
			proxyPort
			);
		
		aContext.createResponseFrom(ca);
	}

}
