package org.netkernel.client.http.endpoint;

import org.netkernel.client.http.representation.HttpCacheRepresentation;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class SimpleInMemoryCache extends StandardAccessorImpl
{	private Map<String, CacheEntry> mMap;
	
	private int mReapCount;
	private static int mMaxReapCount=25;
	private static double mMaxSize=5E6;
	private double mCacheSize;

	public SimpleInMemoryCache()
	{	mMap=new ConcurrentHashMap<String, CacheEntry>(64);		
	}

	@Override
	public void onSink(INKFRequestContext aContext) throws Exception
	{	String url=aContext.getThisRequest().getArgumentValue("url");
		HttpCacheRepresentation rep=(HttpCacheRepresentation)aContext.sourcePrimary(HttpCacheRepresentation.class);
		mMap.put(getCacheKey(url), new CacheEntry(rep));
		mCacheSize+=rep.getContentLength();
		mReapCount++;
		if(mReapCount==mMaxReapCount || mCacheSize>mMaxSize)
		{	reap();
		}
	}

	@Override
	public void onSource(INKFRequestContext aContext) throws Exception
	{	Object result=null;
		String mimetype=null;
		String url=aContext.getThisRequest().getArgumentValue("url");
		CacheEntry e=mMap.get(getCacheKey(url));
		if(e!=null)
		{	result=e.getRepresentation();
		}
		INKFResponse resp=aContext.createResponseFrom(result);
		resp.setMimeType(mimetype);
		if(result==null)
		{	resp.setNoCache();			
		}
	}
	
	private String getCacheKey(String url)
	{	return Integer.toHexString(url.hashCode());
	}
	
	private void reap()
	{	mReapCount=0;
		mMap.clear();
	}
	
	
	private class CacheEntry
	{	long mLastTouch;
		HttpCacheRepresentation mRep;
		int mHits=1;
	
		public CacheEntry(HttpCacheRepresentation aRep)
		{	mRep=aRep;
			touch();
		}
		
		public void touch()
		{	mLastTouch=System.currentTimeMillis();
		}
		
		public HttpCacheRepresentation getRepresentation()
		{	touch();
			mHits++;
			return mRep;
		}
		
		public long getScore()
		{	return (System.currentTimeMillis()-mLastTouch+mRep.getContentLength())/mHits;			
		}
	}
	

}
